with text_io;
use text_io;

package body Mengen is

   type BoolArray is array (Integer range <>) of Boolean; 
   
   type Menge_Rec (von, bis : Integer) is record
   
      chf : BoolArray (von .. bis) := (others => false);
      
   end record;
  
   ------------------------------------------------------------- 

   -- Erstellt eine leere Menge auf der Grundmenge [von .. bis].
   function LeerMengeAuf (von, bis : Integer) return Menge is
   begin
      return new Menge_Rec (von, bis);
   end LeerMengeAuf; 

   -- Erstellt eine Kopie von M.
   function kopieVon (M : Menge) return Menge is
   begin
      return new Menge_Rec'(M.von, M.bis, M.chf);
   end kopieVon;
  
   ------------------------------------------------------------- 

   -- Trgt ein neues Element E in die Menge M ein. 
   procedure Eintrag (E : Integer; M : in out Menge) is
   begin
      if not (E in M.chf'range) then
         raise ElementNichtInGrundmenge;
      else 
         M.chf(E) := True;
      end if;
   end Eintrag;

   -- Lscht ein Element E aus der Menge M.
   procedure Loesche (E : Integer; M : in out Menge) is
   begin
      if not (E in M.chf'range) then
         raise ElementNichtInGrundmenge;
      else 
         M.chf(E) := False;
      end if;
   end Loesche; 
  
   ------------------------------------------------------------- 
  
   -- Trgt alle Elemente in die Menge M ein. 
   procedure Eintragen (Elemente : IntArray; M : in out Menge) is
   begin
      for i in Elemente'range loop
         Eintrag (Elemente(i), M);
      end loop;
   end Eintragen;

   ------------------------------------------------------------- 

   -- Prft, ob E in M liegt.
   function LiegtIn (E : Integer; M : Menge) return Boolean is
   begin
      if not (E in M.chf'range) then
         raise ElementNichtInGrundmenge;
      else 
         return M.chf(E);
      end if;
   end LiegtIn; 
   

   -- Prft, ob eine M leer ist.
   function IstLeer (M : Menge) return Boolean is
   begin
      for i in M.chf'range loop
         if M.chf(i) then return False; end if;
      end loop;
      return True;
   end IstLeer; 
  
   ------------------------------------------------------------- 

   -- Bildet die Vereinigungsmenge zweier Mengen.
   function "+" (M1, M2 : Menge) return Menge is
      M : Menge := LeerMengeAuf (M1.von, M1.bis);
   begin
      if (M1.von /= M2.von or M1.bis /= M2.bis) then
         raise GrundmengenVerschieden;
      else
         for i in M.chf'range loop
            M.chf(i) := M1.chf(i) or M2.chf(i);
         end loop;
         return M;
      end if;
   end "+"; 
 
   -- Bildet die Schnittmenge zweier Mengen.
   function "*" (M1, M2 : Menge) return Menge is
      M : Menge := LeerMengeAuf (M1.von, M1.bis);
   begin
      if (M1.von /= M2.von or M1.bis /= M2.bis) then
         raise GrundmengenVerschieden;
      else
         for i in M.chf'range loop
            M.chf(i) := M1.chf(i) and M2.chf(i);
         end loop;
         return M;
      end if;
   end "*"; 
  
   ------------------------------------------------------------- 

   -- Kleinstes Element der Obermenge von M.
   function von (M : Menge) return Integer is
   begin
      return M.von;
   end von; 

   -- Grtes Element der Obermenge von M.
   function bis (M : Menge) return Integer is
   begin
      return M.bis;
   end bis; 
  
   -------------------------------------------------------------

   -- Nimmt Elemente von M2 zu M1 hinzu.
   procedure Hinzu (M1 : in out Menge; M2 : Menge) is
   begin
      if (M1.von /= M2.von or M1.bis /= M2.bis) then
         raise GrundmengenVerschieden;
      else
         for i in M1.chf'range loop
            M1.chf(i) := M1.chf(i) or M2.chf(i);
         end loop;
      end if;
   end Hinzu; 
 
   -- Entfernt die Elemente von M2 aus M1.
   procedure Hinweg (M1 : in out Menge; M2 : Menge) is
   begin
      if (M1.von /= M2.von or M1.bis /= M2.bis) then
         raise GrundmengenVerschieden;
      else
         for i in M1.chf'range loop
            M1.chf(i) := M1.chf(i) and not M2.chf(i);
         end loop;
      end if;
   end Hinweg; 
  
   ------------------------------------------------------------- 

   -- Druckt M.
   procedure Drucke (M : Menge) is
   begin
      put ("{");
      for i in M.chf'range loop
         if M.chf(i) then put (Integer'image(i)); end if;
      end loop;
      put (" }");
   end Drucke; 

end Mengen;
